@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2022  Free Software Foundation, Inc.
+ * Copyright (C) 2012-2023  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -25,6 +25,8 @@
 
 #if _MIPS_SIM != _ABIO32
 #    define NEW_ABI		1
+#else
+#    define NEW_ABI		0
 #endif
 
 /*
@@ -114,4 +116,23 @@ typedef enum {
     _NOREG,
 } jit_reg_t;
 
+typedef struct {
+    jit_uint32_t release	: 4;
+    /* set if lwc1, ldc1, swc1, sdc1, mtc1, mfc1, dmtc1, and dmfc1
+     * can be put in delay slot */
+    jit_uint32_t cop1_delay	: 1;
+    /* set if sll can be put in delay slot */
+    jit_uint32_t sll_delay	: 1;
+    /* set if lwl and lwr can be put in delay slot */
+    jit_uint32_t lwl_lwr_delay	: 1;
+    /* generate special instructions for un{ld,st}*; ignored for mips 6
+     * that does not have unaligned load/store instructions. */
+    jit_uint32_t unaligned	: 1;
+} jit_cpu_t;
+
+/*
+ * Initialization
+ */
+extern jit_cpu_t		jit_cpu;
+
 #endif /* _jit_mips_h */
