@@ -1352,6 +1352,24 @@ _emit_code(jit_state_t *_jit)
 					rn(node->v.q.h), rn(node->w.w));\
 	    case jit_code_##name##i##type:				\
 		break
+#define case_rrx(name, type)						\
+	    case jit_code_##name##i##type:				\
+		name##i##type(rn(node->u.w), rn(node->v.w), node->w.w);	\
+	       break
+#define case_rrX(name, type)						\
+	    case jit_code_##name##r##type:				\
+		name##r##type(rn(node->u.w),				\
+			      rn(node->v.w), rn(node->w.w));		\
+		break
+#define case_xrr(name, type)						\
+		case jit_code_##name##i##type:				\
+		name##i##type(node->u.w, rn(node->v.w), rn(node->w.w));	\
+		break
+#define case_Xrr(name, type)						\
+	    case jit_code_##name##r##type:				\
+		name##r##type(rn(node->u.w), rn(node->v.w),		\
+			      rn(node->w.w));				\
+		break
 #define case_rrrr(name, type)						\
 	    case jit_code_##name##r##type:				\
 		name##r##type(rn(node->u.q.l), rn(node->u.q.h),		\
@@ -1573,6 +1591,10 @@ _emit_code(jit_state_t *_jit)
 		case_rrw(rsb,);
 		case_rrr(mul,);
 		case_rrw(mul,);
+		case_rrr(hmul,);
+		case_rrw(hmul,);
+		case_rrr(hmul, _u);
+		case_rrw(hmul, _u);
 		case_rrrr(qmul,);
 		case_rrrw(qmul,);
 		case_rrrr(qmul, _u);
@@ -1663,6 +1685,68 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_unldi_u:
 		unldi_u(rn(node->u.w), node->v.w, node->w.w);
 		break;
+		case_rrx(ldxb, _c);	case_rrX(ldxb, _c);
+		case_rrx(ldxa, _c);	case_rrX(ldxa, _c);
+		case_rrx(ldxb, _uc);	case_rrX(ldxb, _uc);
+		case_rrx(ldxa, _uc);	case_rrX(ldxa, _uc);
+		case_rrx(ldxb, _s);	case_rrX(ldxb, _s);
+		case_rrx(ldxa, _s);	case_rrX(ldxa, _s);
+		case_rrx(ldxb, _us);	case_rrX(ldxb, _us);
+		case_rrx(ldxa, _us);	case_rrX(ldxa, _us);
+		case_rrx(ldxb, _i);	case_rrX(ldxb, _i);
+		case_rrx(ldxa, _i);	case_rrX(ldxa, _i);
+	    case jit_code_ldxbr_f:
+		addr(rn(node->v.w), rn(node->v.w), rn(node->w.w));
+		goto L_ldxbi_f;
+	    case jit_code_ldxbi_f:
+		addi(rn(node->v.w), rn(node->v.w), node->w.w);
+	    L_ldxbi_f:
+		if (jit_swf_p())
+		    swf_ldr_f(rn(node->u.w), rn(node->v.w));
+		else
+		    vfp_ldr_f(rn(node->u.w), rn(node->v.w));
+		break;
+	    case jit_code_ldxar_f:
+		if (jit_swf_p())
+		    swf_ldr_f(rn(node->u.w), rn(node->v.w));
+		else
+		    vfp_ldr_f(rn(node->u.w), rn(node->v.w));
+		addr(rn(node->v.w), rn(node->v.w), rn(node->w.w));
+		break;
+	    case jit_code_ldxai_f:
+		if (jit_swf_p()) {
+		    swf_ldr_f(rn(node->u.w), rn(node->v.w));
+		    addi(rn(node->v.w), rn(node->v.w), node->w.w);
+		}
+		else
+		    vfp_ldxai_f(rn(node->u.w), rn(node->v.w), node->w.w);
+		break;
+	    case jit_code_ldxbr_d:
+		addr(rn(node->v.w), rn(node->v.w), rn(node->w.w));
+		goto L_ldxbi_d;
+	    case jit_code_ldxbi_d:
+		addi(rn(node->v.w), rn(node->v.w), node->w.w);
+	    L_ldxbi_d:
+		if (jit_swf_p())
+		    swf_ldr_d(rn(node->u.w), rn(node->v.w));
+		else
+		    vfp_ldr_d(rn(node->u.w), rn(node->v.w));
+		break;
+	    case jit_code_ldxar_d:
+		if (jit_swf_p())
+		    swf_ldr_d(rn(node->u.w), rn(node->v.w));
+		else
+		    vfp_ldr_d(rn(node->u.w), rn(node->v.w));
+		addr(rn(node->v.w), rn(node->v.w), rn(node->w.w));
+		break;
+	    case jit_code_ldxai_d:
+		if (jit_swf_p()) {
+		    swf_ldr_d(rn(node->u.w), rn(node->v.w));
+		    addi(rn(node->v.w), rn(node->v.w), node->w.w);
+		}
+		else
+		    vfp_ldxai_d(rn(node->u.w), rn(node->v.w), node->w.w);
+		break;
 		case_rr(st, _c);
 		case_wr(st, _c);
 		case_rr(st, _s);
@@ -1681,6 +1765,64 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_unsti:
 		unsti(node->u.w, rn(node->v.w), node->w.w);
 		break;
+		case_xrr(stxb, _c);	case_Xrr(stxb, _c);
+		case_xrr(stxa, _c);	case_Xrr(stxa, _c);
+		case_xrr(stxb, _s);	case_Xrr(stxb, _s);
+		case_xrr(stxa, _s);	case_Xrr(stxa, _s);
+		case_xrr(stxb, _i);	case_Xrr(stxb, _i);
+		case_xrr(stxa, _i);	case_Xrr(stxa, _i);
+	    case jit_code_stxbr_f:
+		addr(rn(node->v.w), rn(node->v.w), rn(node->u.w));
+		goto L_stxbi_f;
+	    case jit_code_stxbi_f:
+		addi(rn(node->v.w), rn(node->v.w), node->u.w);
+	    L_stxbi_f:
+		if (jit_swf_p())
+		    swf_str_f(rn(node->v.w), rn(node->w.w));
+		else
+		    vfp_str_f(rn(node->v.w), rn(node->w.w));
+		break;
+	    case jit_code_stxar_f:
+		if (jit_swf_p())
+		    swf_str_f(rn(node->v.w), rn(node->w.w));
+		else
+		    vfp_str_f(rn(node->v.w), rn(node->w.w));
+		addr(rn(node->v.w), rn(node->v.w), rn(node->u.w));
+		break;
+	    case jit_code_stxai_f:
+		if (jit_swf_p()) {
+		    swf_str_f(rn(node->v.w), rn(node->w.w));
+		    addi(rn(node->v.w), rn(node->v.w), node->u.w);
+		}
+		else
+		    vfp_stxai_f(node->u.w, rn(node->v.w), rn(node->w.w));
+		break;
+	    case jit_code_stxbr_d:
+		addr(rn(node->v.w), rn(node->v.w), rn(node->u.w));
+		goto L_stxbi_d;
+	    case jit_code_stxbi_d:
+		addi(rn(node->v.w), rn(node->v.w), node->u.w);
+	    L_stxbi_d:
+		if (jit_swf_p())
+		    swf_str_d(rn(node->v.w), rn(node->w.w));
+		else
+		    vfp_str_d(rn(node->v.w), rn(node->w.w));
+		break;
+	    case jit_code_stxar_d:
+		if (jit_swf_p())
+		    swf_str_d(rn(node->v.w), rn(node->w.w));
+		else
+		    vfp_str_d(rn(node->v.w), rn(node->w.w));
+		addr(rn(node->v.w), rn(node->v.w), rn(node->u.w));
+		break;
+	    case jit_code_stxai_d:
+		if (jit_swf_p()) {
+		    swf_str_d(rn(node->v.w), rn(node->w.w));
+		    addi(rn(node->v.w), rn(node->v.w), node->u.w);
+		}
+		else
+		    vfp_stxai_d(node->u.w, rn(node->v.w), rn(node->w.w));
+		break;
 		case_rr(hton, _us);
 		case_rr(hton, _ui);
 		case_rr(bswap, _us);
@@ -2323,6 +2465,10 @@ _emit_code(jit_state_t *_jit)
 #undef case_vvw
 #undef case_rrw
 #undef case_vvv
+#undef case_rrx
+#undef case_rrX
+#undef case_xrr
+#undef case_Xrr
 #undef case_rrr
 #undef case_wv
 #undef case_wr
@@ -2349,7 +2495,7 @@ _emit_code(jit_state_t *_jit)
 	     * FIXME can this cause issues in the preprocessor prefetch
 	     * or something else? should not, as the constants are after
 	     * an unconditional jump */
-	    if (value & ARM_P)	value =   value & 0x00000fff;
+	    if (value & ARM_U)	value =   value & 0x00000fff;
 	    else		value = -(value & 0x00000fff);
 	    word = word + 8 + value;
  	}
