@@ -524,4 +524,68 @@
     0,	/* fnmai_d */
     12, /* fnmsr_d */
     0,	/* fnmsi_d */
+    36,	/* hmulr */
+    40,	/* hmuli */
+    48,	/* hmulr_u */
+    56,	/* hmuli_u */
+    12, /* ldxbr_c */
+    12, /* ldxbi_c */
+    12, /* ldxar_c */
+    12, /* ldxai_c */
+    8,	/* ldxbr_uc */
+    8,	/* ldxbi_uc */
+    8,	/* ldxar_uc */
+    8,	/* ldxai_uc */
+    12, /* ldxbr_s */
+    12, /* ldxbi_s */
+    12, /* ldxar_s */
+    12, /* ldxai_s */
+    8,	/* ldxbr_us */
+    8,	/* ldxbi_us */
+    8,	/* ldxar_us */
+    8,	/* ldxai_us */
+    8,	/* ldxbr_i */
+    8,	/* ldxbi_i */
+    8,	/* ldxar_i */
+    8,	/* ldxai_i */
+    0,	/* ldxbr_ui */
+    0,	/* ldxbi_ui */
+    0,	/* ldxar_ui */
+    0,	/* ldxai_ui */
+    0,	/* ldxbr_l */
+    0,	/* ldxbi_l */
+    0,	/* ldxar_l */
+    0,	/* ldxai_l */
+    8,	/* ldxbr_f */
+    8,	/* ldxbi_f */
+    8,	/* ldxar_f */
+    8,	/* ldxai_f */
+    8,	/* ldxbr_d */
+    8,	/* ldxbi_d */
+    8,	/* ldxar_d */
+    8,	/* ldxai_d */
+    8,	/* stxbr_c */
+    8,	/* stxbi_c */
+    8,	/* stxar_c */
+    8,	/* stxai_c */
+    8,	/* stxbr_s */
+    8,	/* stxbi_s */
+    8,	/* stxar_s */
+    8,	/* stxai_s */
+    8,	/* stxbr_i */
+    8,	/* stxbi_i */
+    8,	/* stxar_i */
+    8,	/* stxai_i */
+    0,	/* stxbr_l */
+    0,	/* stxbi_l */
+    0,	/* stxar_l */
+    0,	/* stxai_l */
+    8,	/* stxbr_f */
+    8,	/* stxbi_f */
+    8,	/* stxar_f */
+    8,	/* stxai_f */
+    8,	/* stxbr_d */
+    8,	/* stxbi_d */
+    8,	/* stxar_d */
+    8,	/* stxai_d */
 #endif /* __WORDSIZE */
