@@ -912,6 +912,26 @@ _emit_code(jit_state_t *_jit)
 			      rn(node->v.q.h), rn(node->w.w));		\
 	    case jit_code_##name##i##type:				\
 		break;
+#define case_rrx(name, type)						\
+	    case jit_code_##name##i##type:				\
+		generic_##name##i##type(rn(node->u.w),			\
+					rn(node->v.w), node->w.w);	\
+	       break
+#define case_rrX(name, type)						\
+	    case jit_code_##name##r##type:				\
+		generic_##name##r##type(rn(node->u.w),			\
+					rn(node->v.w), rn(node->w.w));	\
+		break
+#define case_xrr(name, type)						\
+		case jit_code_##name##i##type:				\
+		generic_##name##i##type(node->u.w, rn(node->v.w),	\
+					rn(node->w.w));			\
+		break
+#define case_Xrr(name, type)						\
+	    case jit_code_##name##r##type:				\
+		generic_##name##r##type(rn(node->u.w), rn(node->v.w),	\
+					rn(node->w.w));			\
+		break
 #define case_rrrr(name, type)						\
 	    case jit_code_##name##r##type:				\
 		name##r##type(rn(node->u.q.l), rn(node->u.q.h),		\
@@ -1024,6 +1044,10 @@ _emit_code(jit_state_t *_jit)
 		case_rrw(rsb,);
 		case_rrr(mul,);
 		case_rrw(mul,);
+		case_rrr(hmul,);
+		case_rrw(hmul,);
+		case_rrr(hmul, _u);
+		case_rrw(hmul, _u);
 		case_rrrr(qmul,);
 		case_rrrw(qmul,);
 		case_rrrr(qmul, _u);
@@ -1191,6 +1215,20 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_unldi_u:
 		unldi_u(rn(node->u.w), node->v.w, node->w.w);
 		break;
+		case_rrx(ldxb, _c);	case_rrX(ldxb, _c);
+		case_rrx(ldxa, _c);	case_rrX(ldxa, _c);
+		case_rrx(ldxb, _uc);	case_rrX(ldxb, _uc);
+		case_rrx(ldxa, _uc);	case_rrX(ldxa, _uc);
+		case_rrx(ldxb, _s);	case_rrX(ldxb, _s);
+		case_rrx(ldxa, _s);	case_rrX(ldxa, _s);
+		case_rrx(ldxb, _us);	case_rrX(ldxb, _us);
+		case_rrx(ldxa, _us);	case_rrX(ldxa, _us);
+		case_rrx(ldxb, _i);	case_rrX(ldxb, _i);
+		case_rrx(ldxa, _i);	case_rrX(ldxa, _i);
+		case_rrx(ldxb, _f);	case_rrX(ldxb, _f);
+		case_rrx(ldxa, _f);	case_rrX(ldxa, _f);
+		case_rrx(ldxb, _d);	case_rrX(ldxb, _d);
+		case_rrx(ldxa, _d);	case_rrX(ldxa, _d);
 		case_rr(st, _c);
 		case_wr(st, _c);
 		case_rr(st, _s);
@@ -1211,6 +1249,16 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_unsti:
 		unsti(node->u.w, rn(node->v.w), node->w.w);
 		break;
+		case_xrr(stxb, _c);	case_Xrr(stxb, _c);
+		case_xrr(stxa, _c);	case_Xrr(stxa, _c);
+		case_xrr(stxb, _s);	case_Xrr(stxb, _s);
+		case_xrr(stxa, _s);	case_Xrr(stxa, _s);
+		case_xrr(stxb, _i);	case_Xrr(stxb, _i);
+		case_xrr(stxa, _i);	case_Xrr(stxa, _i);
+		case_xrr(stxb, _f);	case_rrX(stxb, _f);
+		case_xrr(stxa, _f);	case_rrX(stxa, _f);
+		case_xrr(stxb, _d);	case_rrX(stxb, _d);
+		case_xrr(stxa, _d);	case_rrX(stxa, _d);
 		case_brr(blt,);
 		case_brw(blt,);
 		case_brr(blt, _u);
@@ -1684,6 +1732,10 @@ _emit_code(jit_state_t *_jit)
 #undef case_rrrw
 #undef case_rrw
 #undef case_rrrr
+#undef case_xrr
+#undef case_Xrr
+#undef case_rrx
+#undef case_rrX
 #undef case_rrr
 #undef case_wr
 #undef case_rw
