@@ -1424,6 +1424,26 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_##name##i##type:				\
 		name##i##type(rn(node->u.w), rn(node->v.w), node->w.w);	\
 		break
+#define case_rrx(name, type)						\
+	    case jit_code_##name##i##type:				\
+		generic_##name##i##type(rn(node->u.w),			\
+					rn(node->v.w), node->w.w);	\
+	       break
+#define case_rrX(name, type)						\
+	    case jit_code_##name##r##type:				\
+		generic_##name##r##type(rn(node->u.w),			\
+					rn(node->v.w), rn(node->w.w));	\
+		break
+#define case_xrr(name, type)						\
+		case jit_code_##name##i##type:				\
+		generic_##name##i##type(node->u.w, rn(node->v.w),	\
+					rn(node->w.w));			\
+		break
+#define case_Xrr(name, type)						\
+	    case jit_code_##name##r##type:				\
+		generic_##name##r##type(rn(node->u.w), rn(node->v.w),	\
+					rn(node->w.w));			\
+		break
 #define case_rrrr(name, type)						\
 	    case jit_code_##name##r##type:				\
 		name##r##type(rn(node->u.q.l), rn(node->u.q.h),		\
@@ -1551,6 +1571,10 @@ _emit_code(jit_state_t *_jit)
 		case_rrw(rsb,);
 		case_rrr(mul,);
 		case_rrw(mul,);
+		case_rrr(hmul,);
+		case_rrw(hmul,);
+		case_rrr(hmul, _u);
+		case_rrw(hmul, _u);
 		case_rrrr(qmul,);
 		case_rrrw(qmul,);
 		case_rrrr(qmul, _u);
@@ -1641,6 +1665,26 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_unldi_u:
 		unldi_u(rn(node->u.w), node->v.w, node->w.w);
 		break;
+		case_rrx(ldxb, _c);	case_rrX(ldxb, _c);
+		case_rrx(ldxa, _c);	case_rrX(ldxa, _c);
+		case_rrx(ldxb, _uc);	case_rrX(ldxb, _uc);
+		case_rrx(ldxa, _uc);	case_rrX(ldxa, _uc);
+		case_rrx(ldxb, _s);	case_rrX(ldxb, _s);
+		case_rrx(ldxa, _s);	case_rrX(ldxa, _s);
+		case_rrx(ldxb, _us);	case_rrX(ldxb, _us);
+		case_rrx(ldxa, _us);	case_rrX(ldxa, _us);
+		case_rrx(ldxb, _i);	case_rrX(ldxb, _i);
+		case_rrx(ldxa, _i);	case_rrX(ldxa, _i);
+#if __WORDSIZE == 64
+		case_rrx(ldxb, _ui);	case_rrX(ldxb, _ui);
+		case_rrx(ldxa, _ui);	case_rrX(ldxa, _ui);
+		case_rrx(ldxb, _l);	case_rrX(ldxb, _l);
+		case_rrx(ldxa, _l);	case_rrX(ldxa, _l);
+#endif
+		case_rrx(ldxb, _f);	case_rrX(ldxb, _f);
+		case_rrx(ldxa, _f);	case_rrX(ldxa, _f);
+		case_rrx(ldxb, _d);	case_rrX(ldxb, _d);
+		case_rrx(ldxa, _d);	case_rrX(ldxa, _d);
 		case_rr(st, _c);
 		case_wr(st, _c);
 		case_rr(st, _s);
@@ -1667,6 +1711,21 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_unsti:
 		unsti(node->u.w, rn(node->v.w), node->w.w);
 		break;
+
+		case_xrr(stxb, _c);	case_Xrr(stxb, _c);
+		case_xrr(stxa, _c);	case_Xrr(stxa, _c);
+		case_xrr(stxb, _s);	case_Xrr(stxb, _s);
+		case_xrr(stxa, _s);	case_Xrr(stxa, _s);
+		case_xrr(stxb, _i);	case_Xrr(stxb, _i);
+		case_xrr(stxa, _i);	case_Xrr(stxa, _i);
+#if __WORDSIZE == 64
+		case_xrr(stxb, _l);	case_rrX(stxb, _l);
+		case_xrr(stxa, _l);	case_rrX(stxa, _l);
+#endif
+		case_xrr(stxb, _f);	case_rrX(stxb, _f);
+		case_xrr(stxa, _f);	case_rrX(stxa, _f);
+		case_xrr(stxb, _d);	case_rrX(stxb, _d);
+		case_xrr(stxa, _d);	case_rrX(stxa, _d);
 		case_rr(hton, _us);
 		case_rr(hton, _ui);
 #if __WORDSIZE == 64
@@ -2111,7 +2170,7 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_movi_w_f:
 		movi_w_f(rn(node->u.w), node->v.w);
 		break;
-#if NEW_ABI
+#if __WORDSIZE == 64 || NEW_ABI
 	    case jit_code_movr_d_w:
 		movr_d_w(rn(node->u.w), rn(node->v.w));
 		break;
@@ -2125,7 +2184,8 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_movi_w_d:
 		movi_w_d(rn(node->u.w), node->v.w);
 		break;
-#else
+#endif
+#if __WORDSIZE == 32
 	    case jit_code_movr_ww_d:
 		movr_ww_d(rn(node->u.w), rn(node->v.w), rn(node->w.w));
 		break;
@@ -2315,6 +2375,10 @@ _emit_code(jit_state_t *_jit)
 #undef case_brr
 #undef case_wrr
 #undef case_rrf
+#undef case_xrr
+#undef case_Xrr
+#undef case_rrx
+#undef case_rrX
 #undef case_rrw
 #undef case_rrr
 #undef case_wr
