@@ -289,4 +289,36 @@ static char *code_name[] = {
     "fnmsr_f",		"fnmsi_f",
     "fnmar_d",		"fnmai_d",
     "fnmsr_d",		"fnmsi_d",
+    "hmulr",		"hmuli",
+    "hmulr_u",		"hmuli_u",
+    "ldxbr_c",		"ldxbi_c",
+    "ldxar_c",		"ldxai_c",
+    "ldxbr_uc",		"ldxbi_uc",
+    "ldxar_uc",		"ldxai_uc",
+    "ldxbr_s",		"ldxbi_s",
+    "ldxar_s",		"ldxai_s",
+    "ldxbr_us",		"ldxbi_us",
+    "ldxar_us",		"ldxai_us",
+    "ldxbr_i",		"ldxbi_i",
+    "ldxar_i",		"ldxai_i",
+    "ldxbr_ui",		"ldxbi_ui",
+    "ldxar_ui",		"ldxai_ui",
+    "ldxbr_l",		"ldxbi_l",
+    "ldxar_l",		"ldxai_l",
+    "ldxbr_f",		"ldxbi_f",
+    "ldxar_f",		"ldxai_f",
+    "ldxbr_d",		"ldxbi_d",
+    "ldxar_d",		"ldxai_d",
+    "stxbr_c",		"stxbi_c",
+    "stxar_c",		"stxai_c",
+    "stxbr_s",		"stxbi_s",
+    "stxar_s",		"stxai_s",
+    "stxbr_i",		"stxbi_i",
+    "stxar_i",		"stxai_i",
+    "stxbr_l",		"stxbi_l",
+    "stxar_l",		"stxai_l",
+    "stxbr_f",		"stxbi_f",
+    "stxar_f",		"stxai_f",
+    "stxbr_d",		"stxbi_d",
+    "stxar_d",		"stxai_d",
 };
