@@ -401,6 +401,18 @@ static void _ldi_f(jit_state_t*,jit_int32_t,jit_word_t);
 static void _ldxr_f(jit_state_t*,jit_int32_t,jit_int32_t,jit_int32_t);
 #  define ldxi_f(r0,r1,i0)		_ldxi_f(_jit,r0,r1,i0)
 static void _ldxi_f(jit_state_t*,jit_int32_t,jit_int32_t,jit_word_t);
+#  define ldxbr_f(r0,r1,r2)		_ldxbr_f(_jit,r0,r1,r2)
+static void _ldxbr_f(jit_state_t*,jit_int32_t,jit_int32_t,jit_int32_t);
+#  define ldxbi_f(r0,r1,i0)		_ldxbi_f(_jit,r0,r1,i0)
+static void _ldxbi_f(jit_state_t*,jit_int32_t,jit_int32_t,jit_word_t);
+#  define ldxbr_d(r0,r1,r2)		_ldxbr_d(_jit,r0,r1,r2)
+static void _ldxbr_d(jit_state_t*,jit_int32_t,jit_int32_t,jit_int32_t);
+#  define ldxbi_d(r0,r1,i0)		_ldxbi_d(_jit,r0,r1,i0)
+static void _ldxbi_d(jit_state_t*,jit_int32_t,jit_int32_t,jit_word_t);
+#  define ldxar_f(r0,r1,r2)		generic_ldxar_f(r0,r1,r2)
+#  define ldxai_f(r0,r1,i0)		generic_ldxai_f(r0,r1,i0)
+#  define ldxar_d(r0,r1,r2)		generic_ldxar_d(r0,r1,r2)
+#  define ldxai_d(r0,r1,i0)		generic_ldxai_d(r0,r1,i0)
 #  define str_f(r0,r1)			STFSX(r1, _R0_REGNO, r0)
 #  define sti_f(i0,r0)			_sti_f(_jit,i0,r0)
 static void _sti_f(jit_state_t*,jit_word_t,jit_int32_t);
@@ -422,6 +434,18 @@ static void _sti_d(jit_state_t*,jit_word_t,jit_int32_t);
 static void _stxr_d(jit_state_t*,jit_int32_t,jit_int32_t,jit_int32_t);
 #  define stxi_d(i0,r0,r1)		_stxi_d(_jit,i0,r0,r1)
 static void _stxi_d(jit_state_t*,jit_word_t,jit_int32_t,jit_int32_t);
+#  define stxbr_f(r0,r1,r2)		_stxbr_f(_jit,r0,r1,r2)
+static void _stxbr_f(jit_state_t*,jit_int32_t,jit_int32_t,jit_int32_t);
+#  define stxbi_f(i0,r0,r1)		_stxbi_f(_jit,i0,r0,r1)
+static void _stxbi_f(jit_state_t*,jit_word_t,jit_int32_t,jit_int32_t);
+#  define stxbr_d(r0,r1,r2)		_stxbr_d(_jit,r0,r1,r2)
+static void _stxbr_d(jit_state_t*,jit_int32_t,jit_int32_t,jit_int32_t);
+#  define stxbi_d(i0,r0,r1)		_stxbi_d(_jit,i0,r0,r1)
+static void _stxbi_d(jit_state_t*,jit_word_t,jit_int32_t,jit_int32_t);
+#  define stxar_f(r0,r1,r2)		generic_stxar_f(r0,r1,r2)
+#  define stxai_f(i0,r0,r1)		generic_stxai_f(i0,r0,r1)
+#  define stxar_d(r0,r1,r2)		generic_stxar_d(r0,r1,r2)
+#  define stxai_d(i0,r0,r1)		generic_stxai_d(i0,r0,r1)
 #endif
 
 #if CODE
@@ -467,12 +491,14 @@ _movi_f(jit_state_t *_jit, jit_int32_t r0, jit_float32_t *i0)
     jit_int32_t		 reg;
 
     if (_jitc->no_data) {
+	CHECK_CVT_OFFSET();
+	jit_check_frame();
 	data.f = *i0;
 	reg = jit_get_reg(jit_class_gpr);
 	movi(rn(reg), data.i & 0xffffffff);
-	stxi_i(alloca_offset - 4, _FP_REGNO, rn(reg));
+	stxi_i(CVT_OFFSET + 4, _FP_REGNO, rn(reg));
 	jit_unget_reg(reg);
-	ldxi_f(r0, _FP_REGNO, alloca_offset - 4);
+	ldxi_f(r0, _FP_REGNO, CVT_OFFSET + 4);
     }
     else
 	ldi_f(r0, (jit_word_t)i0);
@@ -499,19 +525,21 @@ _movi_d(jit_state_t *_jit, jit_int32_t r0, jit_float64_t *i0)
     jit_int32_t		 reg;
 
     if (_jitc->no_data) {
+	CHECK_CVT_OFFSET();
+	jit_check_frame();
 	data.d = *i0;
 	reg = jit_get_reg(jit_class_gpr);
 #  if __WORDSIZE == 32
 	movi(rn(reg), data.i[0]);
-	stxi(alloca_offset - 8, _FP_REGNO, rn(reg));
+	stxi(CVT_OFFSET, _FP_REGNO, rn(reg));
 	movi(rn(reg), data.i[1]);
-	stxi(alloca_offset - 4, _FP_REGNO, rn(reg));
+	stxi(CVT_OFFSET + 4, _FP_REGNO, rn(reg));
 #  else
 	movi(rn(reg), data.w);
-	stxi(alloca_offset - 8, _FP_REGNO, rn(reg));
+	stxi(CVT_OFFSET, _FP_REGNO, rn(reg));
 #  endif
 	jit_unget_reg(reg);
-	ldxi_d(r0, _FP_REGNO, alloca_offset - 8);
+	ldxi_d(r0, _FP_REGNO, CVT_OFFSET);
     }
     else
 	ldi_d(r0, (jit_word_t)i0);
@@ -547,13 +575,14 @@ _extr_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1)
 {
 #  if __WORDSIZE == 32
     jit_int32_t		reg, freg, off1, off2;
-
+    CHECK_CVT_OFFSET();
+    jit_check_frame();
 #  if __BYTE_ORDER == __BIG_ENDIAN
-    off1 = alloca_offset - 8;
-    off2 = alloca_offset - 4;
+    off1 = CVT_OFFSET;
+    off2 = CVT_OFFSET + 4;
 #  else
-    off1 = alloca_offset - 4;
-    off2 = alloca_offset - 8;
+    off1 = CVT_OFFSET + 4;
+    off2 = CVT_OFFSET;
 #  endif
 
     reg = jit_get_reg(jit_class_gpr);
@@ -563,17 +592,17 @@ _extr_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1)
     stxi_i(off1, _FP_REGNO, rn(reg));
     movi(rn(reg), 0x80000000);
     stxi_i(off2, _FP_REGNO, rn(reg));
-    ldxi_d(rn(freg), _FP_REGNO, alloca_offset - 8);
+    ldxi_d(rn(freg), _FP_REGNO, CVT_OFFSET);
     xorr(rn(reg), r1, rn(reg));
     stxi_i(off2, _FP_REGNO, rn(reg));
-    ldxi_d(r0, _FP_REGNO, alloca_offset - 8);
+    ldxi_d(r0, _FP_REGNO, CVT_OFFSET);
     subr_d(r0, r0, rn(freg));
 
     jit_unget_reg(reg);
     jit_unget_reg(freg);
 #  else
-    stxi(alloca_offset - 8, _FP_REGNO, r1);
-    ldxi_d(r0, _FP_REGNO, alloca_offset - 8);
+    stxi(CVT_OFFSET, _FP_REGNO, r1);
+    ldxi_d(r0, _FP_REGNO, CVT_OFFSET);
     FCFID(r0, r0);
 #  endif
 }
@@ -584,12 +613,13 @@ _truncr_d_i(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1)
     jit_int32_t		reg;
     reg = jit_get_reg(jit_class_fpr);
     FCTIWZ(rn(reg), r1);
-    /* use reserved 8 bytes area */
-    stxi_d(alloca_offset - 8, _FP_REGNO, rn(reg));
+    CHECK_CVT_OFFSET();
+    jit_check_frame();
+    stxi_d(CVT_OFFSET, _FP_REGNO, rn(reg));
 #  if __BYTE_ORDER == __BIG_ENDIAN
-    ldxi_i(r0, _FP_REGNO, alloca_offset - 4);
+    ldxi_i(r0, _FP_REGNO, CVT_OFFSET + 4);
 #  else
-    ldxi_i(r0, _FP_REGNO, alloca_offset - 8);
+    ldxi_i(r0, _FP_REGNO, CVT_OFFSET);
 #  endif
     jit_unget_reg(reg);
 }
@@ -601,9 +631,10 @@ _truncr_d_l(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1)
     jit_int32_t		reg;
     reg = jit_get_reg(jit_class_fpr);
     FCTIDZ(rn(reg), r1);
-    /* use reserved 8 bytes area */
-    stxi_d(alloca_offset - 8, _FP_REGNO, rn(reg));
-    ldxi(r0, _FP_REGNO, alloca_offset - 8);
+    CHECK_CVT_OFFSET();
+    jit_check_frame();
+    stxi_d(CVT_OFFSET, _FP_REGNO, rn(reg));
+    ldxi(r0, _FP_REGNO, CVT_OFFSET);
     jit_unget_reg(reg);
 }
 #  endif
@@ -1149,6 +1180,56 @@ _ldxi_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_word_t i0)
     }
 }
 
+static void
+_ldxbr_f(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
+{
+    assert(r1 != _R0_REGNO);
+    assert(r0 != r1);
+    LFSUX(r0, r1, r2);
+}
+
+static void
+_ldxbi_f(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_word_t i0)
+{
+    jit_int32_t		reg;
+    if (can_sign_extend_short_p(i0)) {
+	assert(r1 != _R0_REGNO);
+	assert(r0 != r1);
+	LFSU(r0, r1, i0);
+    }
+    else {
+	reg = jit_get_reg(jit_class_gpr);
+	movi(rn(reg), i0);
+	ldxbr_f(r0, r1, rn(reg));
+	jit_unget_reg(reg);
+    }
+}
+
+static void
+_ldxbr_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
+{
+    assert(r1 != _R0_REGNO);
+    assert(r0 != r1);
+    LFDUX(r0, r1, r2);
+}
+
+static void
+_ldxbi_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_word_t i0)
+{
+    jit_int32_t		reg;
+    if (can_sign_extend_short_p(i0)) {
+	assert(r1 != _R0_REGNO);
+	assert(r0 != r1);
+	LFDU(r0, r1, i0);
+    }
+    else {
+	reg = jit_get_reg(jit_class_gpr);
+	movi(rn(reg), i0);
+	ldxbr_d(r0, r1, rn(reg));
+	jit_unget_reg(reg);
+    }
+}
+
 static void
 _sti_f(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0)
 {
@@ -1284,4 +1365,50 @@ _stxi_d(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0, jit_int32_t r1)
 	jit_unget_reg(reg);
     }
 }
+
+static void
+_stxbr_f(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
+{
+    assert(r1 != _R0_REGNO);
+    STFSUX(r2, r1, r0);
+}
+
+static void
+_stxbi_f(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0, jit_int32_t r1)
+{
+    jit_int32_t		reg;
+    if (can_sign_extend_short_p(i0)) {
+	assert(r1 != _R0_REGNO);
+	STFSU(r1, r0, i0);
+    }
+    else {
+	reg = jit_get_reg(jit_class_gpr);
+	movi(rn(reg), i0);
+	stxbr_f(r0, r1, rn(reg));
+	jit_unget_reg(reg);
+    }
+}
+
+static void
+_stxbr_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
+{
+    assert(r1 != _R0_REGNO);
+    STFDUX(r2, r1, r0);
+}
+
+static void
+_stxbi_d(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0, jit_int32_t r1)
+{
+    jit_int32_t		reg;
+    if (can_sign_extend_short_p(i0)) {
+	assert(r1 != _R0_REGNO);
+	STFDU(r1, r0, i0);
+    }
+    else {
+	reg = jit_get_reg(jit_class_gpr);
+	movi(rn(reg), i0);
+	stxbr_d(r0, r1, rn(reg));
+	jit_unget_reg(reg);
+    }
+}
 #endif
