@@ -527,6 +527,70 @@
     0,	/* fnmai_d */
     4,	/* fnmsr_d */
     0,	/* fnmsi_d */
+    4,	/* hmulr */
+    12, /* hmuli */
+    4,	/* hmulr_u */
+    12, /* hmuli_u */
+    8,	/* ldxbr_c */
+    8,	/* ldxbi_c */
+    12, /* ldxar_c */
+    12, /* ldxai_c */
+    4,	/* ldxbr_uc */
+    4,	/* ldxbi_uc */
+    8,	/* ldxar_uc */
+    8,	/* ldxai_uc */
+    4,	/* ldxbr_s */
+    4,	/* ldxbi_s */
+    8,	/* ldxar_s */
+    8,	/* ldxai_s */
+    4,	/* ldxbr_us */
+    4,	/* ldxbi_us */
+    8,	/* ldxar_us */
+    8,	/* ldxai_us */
+    4,	/* ldxbr_i */
+    4,	/* ldxbi_i */
+    8,	/* ldxar_i */
+    8,	/* ldxai_i */
+    0,	/* ldxbr_ui */
+    0,	/* ldxbi_ui */
+    0,	/* ldxar_ui */
+    0,	/* ldxai_ui */
+    0,	/* ldxbr_l */
+    0,	/* ldxbi_l */
+    0,	/* ldxar_l */
+    0,	/* ldxai_l */
+    4,	/* ldxbr_f */
+    4,	/* ldxbi_f */
+    8,	/* ldxar_f */
+    8,	/* ldxai_f */
+    4,	/* ldxbr_d */
+    4,	/* ldxbi_d */
+    8,	/* ldxar_d */
+    8,	/* ldxai_d */
+    4,	/* stxbr_c */
+    4,	/* stxbi_c */
+    8,	/* stxar_c */
+    8,	/* stxai_c */
+    4,	/* stxbr_s */
+    4,	/* stxbi_s */
+    8,	/* stxar_s */
+    8,	/* stxai_s */
+    4,	/* stxbr_i */
+    4,	/* stxbi_i */
+    8,	/* stxar_i */
+    8,	/* stxai_i */
+    0,	/* stxbr_l */
+    0,	/* stxbi_l */
+    0,	/* stxar_l */
+    0,	/* stxai_l */
+    4,	/* stxbr_f */
+    4,	/* stxbi_f */
+    8,	/* stxar_f */
+    8,	/* stxai_f */
+    4,	/* stxbr_d */
+    4,	/* stxbi_d */
+    8,	/* stxar_d */
+    8,	/* stxai_d */
 #endif /* !_CALL_SYSV */
 #endif /* __BYTE_ORDER */
 #endif /* __powerpc__ */
@@ -1061,6 +1125,70 @@
     0,	/* fnmai_d */
     4,	/* fnmsr_d */
     0,	/* fnmsi_d */
+    4,	/* hmulr */
+    12, /* hmuli */
+    4,	/* hmulr_u */
+    12, /* hmuli_u */
+    8,	/* ldxbr_c */
+    8,	/* ldxbi_c */
+    12, /* ldxar_c */
+    12, /* ldxai_c */
+    4,	/* ldxbr_uc */
+    4,	/* ldxbi_uc */
+    8,	/* ldxar_uc */
+    8,	/* ldxai_uc */
+    4,	/* ldxbr_s */
+    4,	/* ldxbi_s */
+    8,	/* ldxar_s */
+    8,	/* ldxai_s */
+    4,	/* ldxbr_us */
+    4,	/* ldxbi_us */
+    8,	/* ldxar_us */
+    8,	/* ldxai_us */
+    4,	/* ldxbr_i */
+    4,	/* ldxbi_i */
+    8,	/* ldxar_i */
+    8,	/* ldxai_i */
+    0,	/* ldxbr_ui */
+    0,	/* ldxbi_ui */
+    0,	/* ldxar_ui */
+    0,	/* ldxai_ui */
+    0,	/* ldxbr_l */
+    0,	/* ldxbi_l */
+    0,	/* ldxar_l */
+    0,	/* ldxai_l */
+    4,	/* ldxbr_f */
+    4,	/* ldxbi_f */
+    8,	/* ldxar_f */
+    8,	/* ldxai_f */
+    4,	/* ldxbr_d */
+    4,	/* ldxbi_d */
+    8,	/* ldxar_d */
+    8,	/* ldxai_d */
+    4,	/* stxbr_c */
+    4,	/* stxbi_c */
+    8,	/* stxar_c */
+    8,	/* stxai_c */
+    4,	/* stxbr_s */
+    4,	/* stxbi_s */
+    8,	/* stxar_s */
+    8,	/* stxai_s */
+    4,	/* stxbr_i */
+    4,	/* stxbi_i */
+    8,	/* stxar_i */
+    8,	/* stxai_i */
+    0,	/* stxbr_l */
+    0,	/* stxbi_l */
+    0,	/* stxar_l */
+    0,	/* stxai_l */
+    4,	/* stxbr_f */
+    4,	/* stxbi_f */
+    8,	/* stxar_f */
+    8,	/* stxai_f */
+    4,	/* stxbr_d */
+    4,	/* stxbi_d */
+    8,	/* stxar_d */
+    8,	/* stxai_d */
 #endif /* _CALL_SYSV */
 #endif /* __BYTE_ORDER */
 #endif /* __powerpc__ */
@@ -1594,6 +1722,70 @@
     0,	/* fnmai_d */
     4,	/* fnmsr_d */
     0,	/* fnmsi_d */
+    4,	/* hmulr */
+    24,	/* hmuli */
+    4,	/* hmulr_u */
+    24,	/* hmuli_u */
+    8,	/* ldxbr_c */
+    8,	/* ldxbi_c */
+    12, /* ldxar_c */
+    12, /* ldxai_c */
+    4,	/* ldxbr_uc */
+    4,	/* ldxbi_uc */
+    8,	/* ldxar_uc */
+    8,	/* ldxai_uc */
+    4,	/* ldxbr_s */
+    4,	/* ldxbi_s */
+    8,	/* ldxar_s */
+    8,	/* ldxai_s */
+    4,	/* ldxbr_us */
+    4,	/* ldxbi_us */
+    8,	/* ldxar_us */
+    8,	/* ldxai_us */
+    8,	/* ldxbr_i */
+    8,	/* ldxbi_i */
+    8,	/* ldxar_i */
+    8,	/* ldxai_i */
+    4,	/* ldxbr_ui */
+    4,	/* ldxbi_ui */
+    8,	/* ldxar_ui */
+    8,	/* ldxai_ui */
+    4,	/* ldxbr_l */
+    4,	/* ldxbi_l */
+    8,	/* ldxar_l */
+    8,	/* ldxai_l */
+    4,	/* ldxbr_f */
+    4,	/* ldxbi_f */
+    8,	/* ldxar_f */
+    8,	/* ldxai_f */
+    4,	/* ldxbr_d */
+    4,	/* ldxbi_d */
+    8,	/* ldxar_d */
+    8,	/* ldxai_d */
+    4,	/* stxbr_c */
+    4,	/* stxbi_c */
+    8,	/* stxar_c */
+    8,	/* stxai_c */
+    4,	/* stxbr_s */
+    4,	/* stxbi_s */
+    8,	/* stxar_s */
+    8,	/* stxai_s */
+    4,	/* stxbr_i */
+    4,	/* stxbi_i */
+    8,	/* stxar_i */
+    8,	/* stxai_i */
+    4,	/* stxbr_l */
+    4,	/* stxbi_l */
+    8,	/* stxar_l */
+    8,	/* stxai_l */
+    4,	/* stxbr_f */
+    4,	/* stxbi_f */
+    8,	/* stxar_f */
+    8,	/* stxai_f */
+    4,	/* stxbr_d */
+    4,	/* stxbi_d */
+    8,	/* stxar_d */
+    8,	/* stxai_d */
 #endif /* __BYTE_ORDER */
 #endif /* __powerpc__ */
 #endif /* __WORDSIZE */
@@ -2126,6 +2318,70 @@
     0,	/* fnmai_d */
     4,	/* fnmsr_d */
     0,	/* fnmsi_d */
+    4,	/* hmulr */
+    24,	/* hmuli */
+    4,	/* hmulr_u */
+    24,	/* hmuli_u */
+    8,	/* ldxbr_c */
+    8,	/* ldxbi_c */
+    12, /* ldxar_c */
+    12, /* ldxai_c */
+    4,	/* ldxbr_uc */
+    4,	/* ldxbi_uc */
+    8,	/* ldxar_uc */
+    8,	/* ldxai_uc */
+    4,	/* ldxbr_s */
+    4,	/* ldxbi_s */
+    8,	/* ldxar_s */
+    8,	/* ldxai_s */
+    4,	/* ldxbr_us */
+    4,	/* ldxbi_us */
+    8,	/* ldxar_us */
+    8,	/* ldxai_us */
+    8,	/* ldxbr_i */
+    8,	/* ldxbi_i */
+    8,	/* ldxar_i */
+    8,	/* ldxai_i */
+    4,	/* ldxbr_ui */
+    4,	/* ldxbi_ui */
+    8,	/* ldxar_ui */
+    8,	/* ldxai_ui */
+    4,	/* ldxbr_l */
+    4,	/* ldxbi_l */
+    8,	/* ldxar_l */
+    8,	/* ldxai_l */
+    4,	/* ldxbr_f */
+    4,	/* ldxbi_f */
+    8,	/* ldxar_f */
+    8,	/* ldxai_f */
+    4,	/* ldxbr_d */
+    4,	/* ldxbi_d */
+    8,	/* ldxar_d */
+    8,	/* ldxai_d */
+    4,	/* stxbr_c */
+    4,	/* stxbi_c */
+    8,	/* stxar_c */
+    8,	/* stxai_c */
+    4,	/* stxbr_s */
+    4,	/* stxbi_s */
+    8,	/* stxar_s */
+    8,	/* stxai_s */
+    4,	/* stxbr_i */
+    4,	/* stxbi_i */
+    8,	/* stxar_i */
+    8,	/* stxai_i */
+    4,	/* stxbr_l */
+    4,	/* stxbi_l */
+    8,	/* stxar_l */
+    8,	/* stxai_l */
+    4,	/* stxbr_f */
+    4,	/* stxbi_f */
+    8,	/* stxar_f */
+    8,	/* stxai_f */
+    4,	/* stxbr_d */
+    4,	/* stxbi_d */
+    8,	/* stxar_d */
+    8,	/* stxai_d */
 #endif /* __BYTE_ORDER */
 #endif /* __powerpc__ */
 #endif /* __WORDSIZE */
