@@ -16,6 +16,12 @@
  * Authors:
  *	Paulo Cesar Pereira de Andrade
  */
+#define CHECK_POPCNTB	0
+
+#if CHECK_POPCNTB
+#include <signal.h>
+#include <setjmp.h>
+#endif
 
 #define jit_arg_reg_p(i)		((i) >= 0 && (i) < 8)
 #if !_CALL_SYSV
@@ -47,6 +53,15 @@
 #  define I_DISP			(__WORDSIZE >> 3) - sizeof(jit_int32_t)
 #  define F_DISP			(__WORDSIZE >> 3) - sizeof(jit_float32_t)
 #endif
+#define CVT_OFFSET			_jitc->function->cvt_offset
+#define CHECK_CVT_OFFSET()						\
+    do {								\
+	if (!_jitc->function->cvt_offset) {				\
+	    _jitc->again = 1;						\
+	    _jitc->function->cvt_offset =				\
+		 jit_allocai(sizeof(jit_float64_t));			\
+	}								\
+    } while (0)
 
 /*
  * Types
@@ -103,6 +118,7 @@ extern void __clear_cache(void *, void *);
 /*
  * Initialization
  */
+jit_cpu_t		jit_cpu;
 jit_register_t		_rvs[] = {
     { rc(sav) | 0,			"r0" },
     { rc(sav) | 11,			"r11" },	/* env */
@@ -178,13 +194,73 @@ jit_register_t		_rvs[] = {
     { rc(arg) | rc(fpr) | 1,		"f1" },
     { _NOREG,				"<none>" },
 };
+#if CHECK_POPCNTB
+static sigjmp_buf	jit_env;
+#endif
+
+static jit_int32_t iregs[] = {
+    _R14, _R15, _R16, _R17, _R18, _R19, _R20, _R21, _R22,
+    _R23, _R24, _R25, _R26, _R27, _R28, _R29, _R30
+};
+
+static jit_int32_t fregs[] = {
+    _F14, _F15, _F16, _F17, _F18, _F19, _F20, _F21,
+};
 
 /*
  * Implementation
  */
+#if CHECK_POPCNTB
+static void
+sigill_handler(int signum)
+{
+    jit_cpu.popcntb = 0;
+    siglongjmp(jit_env, 1);
+}
+#endif
+
 void
 jit_get_cpu(void)
 {
+#if CHECK_POPCNTB
+    long		r12;
+    struct		sigaction new_action, old_action;
+    new_action.sa_handler = sigill_handler;
+    sigemptyset(&new_action.sa_mask);
+    new_action.sa_flags = 0;
+    sigaction(SIGILL, NULL, &old_action);
+    if (old_action.sa_handler != SIG_IGN) {
+	sigaction(SIGILL, &new_action, NULL);
+	if (!sigsetjmp(jit_env, 1)) {
+	    jit_cpu.popcntb = 1;
+	    /* popcntb %r12, %r12 */
+	    __asm__ volatile("mr %%r12, %0;"
+			     "popcntb %%r12, %%r12;"
+			     "mr %0, %%r12;"
+			     : "=r" (r12), "=r" (r12));
+	    sigaction(SIGILL, &old_action, NULL);
+	}
+    }
+#elif defined(__linux__)
+    FILE	*fp;
+    char	*ptr;
+    long	 vers;
+    char	 buf[128];
+
+    if ((fp = fopen("/proc/cpuinfo", "r")) != NULL) {
+	while (fgets(buf, sizeof(buf), fp)) {
+	    if (strncmp(buf, "cpu\t\t: POWER", 12) == 0) {
+		vers = strtol(buf + 12, &ptr, 10);
+		jit_cpu.popcntb = vers > 5;
+		break;
+	    }
+	}
+	fclose(fp);
+    }
+#else
+    /* By default, assume it is not available */
+    jit_cpu.popcntb = 0;
+#endif
 }
 
 void
@@ -241,6 +317,7 @@ jit_int32_t
 _jit_allocai(jit_state_t *_jit, jit_int32_t length)
 {
     assert(_jitc->function);
+    jit_check_frame();
     switch (length) {
 	case 0:	case 1:						break;
 	case 2:		_jitc->function->self.aoff &= -2;	break;
@@ -506,8 +583,10 @@ _jit_getarg_c(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(getarg_c, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_c(u, JIT_RA0 - v->u.w);
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_c(u, JIT_FP, v->u.w + C_DISP);
+    }
     jit_dec_synth();
 }
 
@@ -518,8 +597,10 @@ _jit_getarg_uc(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(getarg_uc, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_uc(u, JIT_RA0 - v->u.w);
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_uc(u, JIT_FP, v->u.w + C_DISP);
+    }
     jit_dec_synth();
 }
 
@@ -530,8 +611,10 @@ _jit_getarg_s(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(getarg_s, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_s(u, JIT_RA0 - v->u.w);
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_s(u, JIT_FP, v->u.w + S_DISP);
+    }
     jit_dec_synth();
 }
 
@@ -542,8 +625,10 @@ _jit_getarg_us(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(getarg_us, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_us(u, JIT_RA0 - v->u.w);
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_us(u, JIT_FP, v->u.w + S_DISP);
+    }
     jit_dec_synth();
 }
 
@@ -559,8 +644,10 @@ _jit_getarg_i(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 	jit_extr_i(u, JIT_RA0 - v->u.w);
 #endif
     }
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_i(u, JIT_FP, v->u.w + I_DISP);
+    }
     jit_dec_synth();
 }
 
@@ -572,8 +659,10 @@ _jit_getarg_ui(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(getarg_ui, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_ui(u, JIT_RA0 - v->u.w);
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_ui(u, JIT_FP, v->u.w + I_DISP);
+    }
     jit_dec_synth();
 }
 
@@ -584,8 +673,10 @@ _jit_getarg_l(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(getarg_l, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(u, JIT_RA0 - v->u.w);
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_l(u, JIT_FP, v->u.w);
+    }
     jit_dec_synth();
 }
 #endif
@@ -597,8 +688,10 @@ _jit_putargr(jit_state_t *_jit, jit_int32_t u, jit_node_t *v, jit_code_t code)
     jit_code_inc_synth_wp(code, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(JIT_RA0 - v->u.w, u);
-    else
+    else {
+	jit_check_frame();
 	jit_stxi(v->u.w, JIT_FP, u);
+    }
     jit_dec_synth();
 }
 
@@ -611,6 +704,7 @@ _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v, jit_code_t code)
     if (jit_arg_reg_p(v->u.w))
 	jit_movi(JIT_RA0 - v->u.w, u);
     else {
+	jit_check_frame();
 	regno = jit_get_reg(jit_class_gpr);
 	jit_movi(regno, u);
 	jit_stxi(v->u.w, JIT_FP, regno);
@@ -626,8 +720,10 @@ _jit_getarg_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(getarg_f, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(u, JIT_FA0 - v->u.w);
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_f(u, JIT_FP, v->u.w);
+    }
     jit_dec_synth();
 }
 
@@ -638,8 +734,10 @@ _jit_putargr_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(putargr_f, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(JIT_FA0 - v->u.w, u);
-    else
+    else {
+	jit_check_frame();
 	jit_stxi_f(v->u.w, JIT_FP, u);
+    }
     jit_dec_synth();
 }
 
@@ -652,6 +750,7 @@ _jit_putargi_f(jit_state_t *_jit, jit_float32_t u, jit_node_t *v)
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movi_d(JIT_FA0 - v->u.w, u);
     else {
+	jit_check_frame();
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_d(regno, u);
 	jit_stxi_f(v->u.w, JIT_FP, regno);
@@ -667,8 +766,10 @@ _jit_getarg_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(getarg_d, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(u, JIT_FA0 - v->u.w);
-    else
+    else {
+	jit_check_frame();
 	jit_ldxi_d(u, JIT_FP, v->u.w);
+    }
     jit_dec_synth();
 }
 
@@ -679,8 +780,10 @@ _jit_putargr_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     jit_inc_synth_wp(putargr_d, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(JIT_FA0 - v->u.w, u);
-    else
+    else {
+	jit_check_frame();
 	jit_stxi_d(v->u.w, JIT_FP, u);
+    }
     jit_dec_synth();
 }
 
@@ -693,6 +796,7 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movi_d(JIT_FA0 - v->u.w, u);
     else {
+	jit_check_frame();
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_d(regno, u);
 	jit_stxi_d(v->u.w, JIT_FP, regno);
@@ -783,14 +887,14 @@ _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 			  + 1
 #  endif
 			   )) {
-	/* use reserved 8 bytes area */
-	jit_stxi_d(alloca_offset - 8, JIT_FP, u);
-	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		 alloca_offset - 8);
+	CHECK_CVT_OFFSET();
+	jit_check_frame();
+	jit_stxi_d(CVT_OFFSET, JIT_FP, u);
+	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP, CVT_OFFSET);
 	_jitc->function->call.argi++;
 #  if __WORDSIZE == 32
 	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		 alloca_offset - 4);
+		 CVT_OFFSET + 4);
 	_jitc->function->call.argi++;
 #  endif
     }
@@ -840,14 +944,15 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 			  + 1
 #  endif
 			  )) {
-	    /* use reserved 8 bytes area */
-	    jit_stxi_d(alloca_offset - 8, JIT_FP, regno);
+	    CHECK_CVT_OFFSET();
+	    jit_check_frame();
+	    jit_stxi_d(CVT_OFFSET, JIT_FP, regno);
 	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		     alloca_offset - 8);
+		     CVT_OFFSET);
 	    _jitc->function->call.argi++;
 #  if __WORDSIZE == 32
 	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		     alloca_offset - 4);
+		     CVT_OFFSET + 4);
 	    _jitc->function->call.argi++;
 #  endif
 	}
@@ -895,14 +1000,13 @@ _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 			  + 1
 #  endif
 			   )) {
-	/* use reserved 8 bytes area */
-	jit_stxi_d(alloca_offset - 8, JIT_FP, u);
-	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		 alloca_offset - 8);
+	CHECK_CVT_OFFSET();
+	jit_check_frame();
+	jit_stxi_d(CVT_OFFSET, JIT_FP, u);
+	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP, CVT_OFFSET);
 	_jitc->function->call.argi++;
 #  if __WORDSIZE == 32
-	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		 alloca_offset - 4);
+	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP, CVT_OFFSET + 4);
 	_jitc->function->call.argi++;
 #  endif
     }
@@ -966,14 +1070,15 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 			  + 1
 #  endif
 			  )) {
-	    /* use reserved 8 bytes area */
-	    jit_stxi_d(alloca_offset - 8, JIT_FP, regno);
+	    CHECK_CVT_OFFSET();
+	    jit_check_frame();
+	    jit_stxi_d(CVT_OFFSET, JIT_FP, regno);
 	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		     alloca_offset - 8);
+		     CVT_OFFSET);
 	    _jitc->function->call.argi++;
 #  if __WORDSIZE == 32
 	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		     alloca_offset - 4);
+		     CVT_OFFSET + 4);
 	    _jitc->function->call.argi++;
 #  endif
 	}
@@ -1215,6 +1320,24 @@ _emit_code(jit_state_t *_jit)
 		name##r##type(rn(node->u.w),				\
 			      rn(node->v.w), rn(node->w.w));		\
 		break
+#define case_rrx(name, type)						\
+	    case jit_code_##name##i##type:				\
+		name##i##type(rn(node->u.w), rn(node->v.w), node->w.w);	\
+	       break
+#define case_rrX(name, type)						\
+	    case jit_code_##name##r##type:				\
+		name##r##type(rn(node->u.w),				\
+			      rn(node->v.w), rn(node->w.w));		\
+		break
+#define case_xrr(name, type)						\
+		case jit_code_##name##i##type:				\
+		name##i##type(node->u.w, rn(node->v.w), rn(node->w.w));	\
+		break
+#define case_Xrr(name, type)						\
+	    case jit_code_##name##r##type:				\
+		name##r##type(rn(node->u.w), rn(node->v.w),		\
+			      rn(node->w.w));				\
+		break
 #define case_rrrr(name, type)						\
 	    case jit_code_##name##r##type:				\
 		name##r##type(rn(node->u.q.l), rn(node->u.q.h),		\
@@ -1330,6 +1453,10 @@ _emit_code(jit_state_t *_jit)
 		case_rrw(rsb,);
 		case_rrr(mul,);
 		case_rrw(mul,);
+		case_rrr(hmul,);
+		case_rrw(hmul,);
+		case_rrr(hmul, _u);
+		case_rrw(hmul, _u);
 		case_rrrr(qmul,);
 		case_rrrw(qmul,);
 		case_rrrr(qmul, _u);
@@ -1553,6 +1680,26 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_unldi_u:
 		unldi_u(rn(node->u.w), node->v.w, node->w.w);
 		break;
+		case_rrx(ldxb, _c);	case_rrX(ldxb, _c);
+		case_rrx(ldxa, _c);	case_rrX(ldxa, _c);
+		case_rrx(ldxb, _uc);	case_rrX(ldxb, _uc);
+		case_rrx(ldxa, _uc);	case_rrX(ldxa, _uc);
+		case_rrx(ldxb, _s);	case_rrX(ldxb, _s);
+		case_rrx(ldxa, _s);	case_rrX(ldxa, _s);
+		case_rrx(ldxb, _us);	case_rrX(ldxb, _us);
+		case_rrx(ldxa, _us);	case_rrX(ldxa, _us);
+		case_rrx(ldxb, _i);	case_rrX(ldxb, _i);
+		case_rrx(ldxa, _i);	case_rrX(ldxa, _i);
+#if __WORDSIZE == 64
+		case_rrx(ldxb, _ui);	case_rrX(ldxb, _ui);
+		case_rrx(ldxa, _ui);	case_rrX(ldxa, _ui);
+		case_rrx(ldxb, _l);	case_rrX(ldxb, _l);
+		case_rrx(ldxa, _l);	case_rrX(ldxa, _l);
+#endif
+		case_rrx(ldxb, _f);	case_rrX(ldxb, _f);
+		case_rrx(ldxa, _f);	case_rrX(ldxa, _f);
+		case_rrx(ldxb, _d);	case_rrX(ldxb, _d);
+		case_rrx(ldxa, _d);	case_rrX(ldxa, _d);
 		case_rr(st, _c);
 		case_wr(st, _c);
 		case_rrr(stx, _c);
@@ -1579,6 +1726,20 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_unsti:
 		unsti(node->u.w, rn(node->v.w), node->w.w);
 		break;
+		case_xrr(stxb, _c);	case_Xrr(stxb, _c);
+		case_xrr(stxa, _c);	case_Xrr(stxa, _c);
+		case_xrr(stxb, _s);	case_Xrr(stxb, _s);
+		case_xrr(stxa, _s);	case_Xrr(stxa, _s);
+		case_xrr(stxb, _i);	case_Xrr(stxb, _i);
+		case_xrr(stxa, _i);	case_Xrr(stxa, _i);
+#if __WORDSIZE == 64
+		case_xrr(stxb, _l);	case_rrX(stxb, _l);
+		case_xrr(stxa, _l);	case_rrX(stxa, _l);
+#endif
+		case_xrr(stxb, _f);	case_rrX(stxb, _f);
+		case_xrr(stxa, _f);	case_rrX(stxa, _f);
+		case_xrr(stxb, _d);	case_rrX(stxb, _d);
+		case_xrr(stxa, _d);	case_rrX(stxa, _d);
 		case_rr(mov, _f);
 	    case jit_code_movi_f:
 		assert(node->flag & jit_flag_data);
@@ -1770,6 +1931,7 @@ _emit_code(jit_state_t *_jit)
 		case_rrr(stx, _d);
 		case_wrr(stx, _d);
 	    case jit_code_jmpr:
+		jit_check_frame();
 		jmpr(rn(node->u.w));
 		break;
 	    case jit_code_jmpi:
@@ -1793,8 +1955,10 @@ _emit_code(jit_state_t *_jit)
 			patch(word, node);
 		    }
 		}
-		else
+		else {
+		    jit_check_frame();
 		    jmpi(node->u.w);
+		}
 		break;
 	    case jit_code_callr:
 #if _CALL_SYSV
@@ -1806,11 +1970,13 @@ _emit_code(jit_state_t *_jit)
 #  define xcalli_p(u, v)	calli_p(u)
 #  define xcalli(u, v)		calli(u)
 #endif
+		jit_check_frame();
 		xcallr(rn(node->u.w), !!(node->flag & jit_flag_varargs));
 		break;
 	    case jit_code_calli:
 		value = !!(node->flag & jit_flag_varargs);
 		if (node->flag & jit_flag_node) {
+		    _jitc->function->need_return = 1;
 		    temp = node->u.n;
 		    assert(temp->code == jit_code_label ||
 			   temp->code == jit_code_epilog);
@@ -1828,8 +1994,10 @@ _emit_code(jit_state_t *_jit)
 			patch(word, node);
 		    }
 		}
-		else
+		else {
+		    jit_check_frame();
 		    xcalli(node->u.w, value);
+		}
 		break;
 	    case jit_code_prolog:
 		_jitc->function = _jitc->functions.ptr + node->w.w;
@@ -1884,6 +2052,9 @@ _emit_code(jit_state_t *_jit)
 		     * the reason of the undo. */
 		    undo.func.self.aoff = _jitc->function->frame +
 			_jitc->function->self.aoff;
+		    undo.func.need_frame = _jitc->function->need_frame;
+		    undo.func.need_stack = _jitc->function->need_stack;
+		    undo.func.need_return = _jitc->function->need_return;
 		    jit_regset_set(&undo.func.regset, &_jitc->function->regset);
 		    /* allocar information also does not need to be undone */
 		    undo.func.aoffoff = _jitc->function->aoffoff;
@@ -2106,6 +2277,10 @@ _emit_code(jit_state_t *_jit)
 #undef case_wrr
 #undef case_rrf
 #undef case_rrw
+#undef case_xrr
+#undef case_Xrr
+#undef case_rrx
+#undef case_rrX
 #undef case_rrr
 #undef case_wr
 #undef case_rw
